==> emencia-crma-2.4.2.tar.gz: emencia-crma-2.4.2/crma/forms.py <==
# -*- coding: utf-8 -*-

import tablib

from django import forms

from django.utils.translation import ugettext_lazy as _

from crma.models import MailingList, Contact


class MailingListAdminForm(forms.ModelForm):
    contacts = forms.FileField(
        required=False,
        help_text=_(u'The CSV file must have a header line with two columns: "email" and "lang".')
    )

    class Meta:
        model = MailingList
        fields = [
            'title',
            'all',
            'members',
        ]

    def clean_contacts(self):
        if not self.cleaned_data['contacts']:
            return None

        dataset = tablib.Dataset()
        value = self.cleaned_data['contacts'].read()
        try:
            value = value.decode('utf-8')
        except UnicodeDecodeError as e:
            pass

        dataset.load(value)
        headers = dataset.headers

        fields = set(f.name for f in Contact._meta.fields)
        fields.discard('id')
        for h in headers:
            if h not in fields:
                raise forms.ValidationError('{} is not a field of Contact'.format(h))

        contacts = []
        for line in dataset:
            try:
                contacts.append(dict(zip(headers, line)))
            except (ValueError, TypeError) as e:
                raise forms.ValidationError(e)
        return contacts

    def save_contacts(self):
        contacts = []
        for contact_def in self.cleaned_data['contacts']:
            contact, created = Contact.objects.update_or_create(
                email=contact_def['email'],
                defaults={
                    'lang': contact_def['lang'],
                })
            contacts.append(contact)
        self.instance.members.add(*contacts)

    def save(self, **kw):
        instance = super(MailingListAdminForm, self).save(**kw)
        if hasattr(self, 'save_m2m'):
            original = self.save_m2m

            def save_m2m():
                original()
                self.save_contacts()

            self.save_m2m = save_m2m
        else:
            self.save_contacts()

        return instance

==> emencia-crma-2.6.2.tar.gz: emencia-crma-2.6.2/crma/forms.py <==
# -*- coding: utf-8 -*-

import tablib

from django import forms

from django.utils.translation import ugettext_lazy as _

from crma.models import MailingList, Contact


class MailingListAdminForm(forms.ModelForm):
    contacts = forms.FileField(
        required=False,
        help_text=_(u'The CSV file must have a header line with two columns: "email" and "lang".')
    )

    class Meta:
        model = MailingList
        fields = [
            'title',
            'all',
            'members',
        ]

    def clean_contacts(self):
        if 'contacts' not in self.cleaned_data:
            return None

        dataset = tablib.Dataset()
        value = self.cleaned_data['contacts'].read()
        try:
            value = value.decode('utf-8')
        except UnicodeDecodeError:
            pass

        dataset.load(value)
        headers = dataset.headers

        fields = set(f.name for f in Contact._meta.fields)
        fields.discard('id')
        for h in headers:
            if h not in fields:
                raise forms.ValidationError('{} is not a field of Contact'.format(h))

        contacts = []
        for line in dataset:
            try:
                contacts.append(dict(zip(headers, line)))
            except (ValueError, TypeError) as e:
                raise forms.ValidationError(e)
        return contacts

    def save_contacts(self):
        if 'contacts' not in self.cleaned_data:
            return
        contacts = []
        for contact_def in self.cleaned_data['contacts']:
            contact, created = Contact.objects.update_or_create(
                email=contact_def['email'],
                defaults={
                    'lang': contact_def['lang'],
                })
            contacts.append(contact)
        self.instance.members.add(*contacts)

    def save(self, **kw):
        instance = super(MailingListAdminForm, self).save(**kw)
        if hasattr(self, 'save_m2m'):
            original = self.save_m2m

            def save_m2m():
                original()
                self.save_contacts()

            self.save_m2m = save_m2m
        else:
            self.save_contacts()

        return instance
